from torch import nn
import torch
import sys
import os

sys.path.append(os.getcwd())
sys.path.append(os.path.dirname(os.path.dirname(os.getcwd())))

from models.eval_scripts.complexity import  all_in_one_train, all_in_one_test
from models.training_structures.MCTN_Level2 import train, test # noqa
from unimodals.common_models import GRU, MLP # noqa
from fusions.MCTN import Encoder, Decoder # noqa
from datasets.affect.get_data import get_dataloader # noqa


# mosi_data.pkl, mosei_senti_data.pkl
# mosi_raw.pkl, mosei_raw.pkl, sarcasm.pkl, humor.pkl
# raw_path: mosi.hdf5, mosei.hdf5, sarcasm_raw_text.pkl, humor_raw_text.pkl
if __name__ == '__main__':
    traindata, validdata, testdata = \
    get_dataloader('C:/Users/29296/Documents/Tencent Files/2929629852/FileRecv/sarcasm.pkl', robust_test=False,data_type='sarcasm')

    max_seq = 20
    feature_dim = 371
    hidden_dim = 32

    encoder0 = Encoder(feature_dim, hidden_dim, n_layers=1, dropout=0.0).cuda()
    decoder0 = Decoder(hidden_dim, feature_dim, n_layers=1, dropout=0.0).cuda()
    encoder1 = Encoder(hidden_dim, hidden_dim, n_layers=1, dropout=0.0).cuda()
    decoder1 = Decoder(hidden_dim, feature_dim, n_layers=1, dropout=0.0).cuda()

    reg_encoder = nn.GRU(hidden_dim, 32).cuda()
    head = MLP(32, 64, 1).cuda()

    allmodules = [encoder0, decoder0, encoder1, decoder1, reg_encoder, head]


    def trainprocess():
     train(
        traindata, validdata,
        encoder0, decoder0, encoder1, decoder1,
        reg_encoder, head,
        criterion_t0=nn.MSELoss(), criterion_c=nn.MSELoss(),
        criterion_t1=nn.MSELoss(), criterion_r=nn.L1Loss(),
        max_seq_len=20,
        mu_t0=0.01, mu_c=0.01, mu_t1=0.01,
        dropout_p=0.15, early_stop=False, patience_num=15,
        lr=1e-4, weight_decay=0.01, op_type=torch.optim.AdamW,
        epoch=100, model_save='best_mctn.pt')


    # 可以使用
    with torch.profiler.profile(
            activities=[
                torch.profiler.ProfilerActivity.CPU,
                torch.profiler.ProfilerActivity.CUDA,
            ]
    ) as p:
        all_in_one_train(trainprocess, allmodules)
    print(p.key_averages().table(
        sort_by="self_cuda_time_total", row_limit=-1))
    #all_in_one_train(trainprocess, allmodules)

    model = torch.load('best_mctn.pt').cuda()

    test(model, testdata, 'mosi', no_robust=True)